"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fs = tslib_1.__importStar(require("fs-extra"));
var path = tslib_1.__importStar(require("path"));
var deep_extends_1 = require("../deep-extends/deep-extends");
var parsing_1 = require("../parsing/parsing");
var getPathAndFormat = function (input) {
    if (typeof input === 'string') {
        return { path: input };
    }
    else if (!input.raw) {
        return {
            path: input.path,
            format: input.format,
        };
    }
    else {
        return { path: null, format: null };
    }
};
var getFileData = function (input, path) {
    if (path) {
        return fs.readFile(path, 'utf-8');
    }
    if (input.raw) {
        return input.raw;
    }
    return null;
};
function cobble(params) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var inputs, disableReadFile, ignoreMissingVariables, debug, objects, _i, inputs_1, input, _a, filePath, format, fileData, e_1, message, docs, i, doc, parsed, e_2, messageArr, message, dirName, resolved, e_3, messageArr, message;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    inputs = params.inputs, disableReadFile = params.disableReadFile, ignoreMissingVariables = params.ignoreMissingVariables;
                    debug = params.debug || (function () { });
                    if (!Array.isArray(inputs)) {
                        throw new Error("Please provide a list of inputs either as strings or { path: string, format: 'json' | 'yaml' | 'properties' } objects");
                    }
                    objects = [];
                    _i = 0, inputs_1 = inputs;
                    _b.label = 1;
                case 1:
                    if (!(_i < inputs_1.length)) return [3, 16];
                    input = inputs_1[_i];
                    _a = getPathAndFormat(input), filePath = _a.path, format = _a.format;
                    fileData = void 0;
                    _b.label = 2;
                case 2:
                    _b.trys.push([2, 4, , 5]);
                    return [4, getFileData(input, filePath)];
                case 3:
                    fileData = _b.sent();
                    return [3, 5];
                case 4:
                    e_1 = _b.sent();
                    message = "Could not read config file: " + e_1.message;
                    if (!ignoreMissingVariables) {
                        throw new Error(message);
                    }
                    debug(message + ", continuing");
                    return [3, 5];
                case 5:
                    if (!fileData) {
                        return [3, 15];
                    }
                    docs = parsing_1.splitYamlIntoDocs(fileData);
                    i = 0;
                    _b.label = 6;
                case 6:
                    if (!(i < docs.length)) return [3, 15];
                    doc = docs[i];
                    parsed = void 0;
                    _b.label = 7;
                case 7:
                    _b.trys.push([7, 9, , 10]);
                    return [4, parsing_1.tryParse(doc, format)];
                case 8:
                    parsed = _b.sent();
                    debug("parsed: " + JSON.stringify(parsed));
                    return [3, 10];
                case 9:
                    e_2 = _b.sent();
                    messageArr = ["Could not parse"];
                    if (docs.length > 1) {
                        messageArr = messageArr.concat("for doc " + i);
                    }
                    messageArr = messageArr.concat(e_2.message);
                    message = messageArr.join(' ');
                    if (!ignoreMissingVariables) {
                        throw new Error(message);
                    }
                    debug(message + ", continuing");
                    return [3, 10];
                case 10:
                    if (disableReadFile) {
                        debug("file inlining is not enabled");
                        objects = objects.concat(parsed);
                        return [3, 14];
                    }
                    _b.label = 11;
                case 11:
                    _b.trys.push([11, 13, , 14]);
                    dirName = filePath ? path.dirname(filePath) : __dirname;
                    debug("dirName at " + dirName);
                    return [4, parsing_1.resolveFiles(parsed, dirName)];
                case 12:
                    resolved = _b.sent();
                    debug("resolved files: " + JSON.stringify(resolved));
                    objects = objects.concat(resolved);
                    return [3, 14];
                case 13:
                    e_3 = _b.sent();
                    messageArr = ["Could not resolve files:"];
                    if (docs.length > 1) {
                        messageArr = messageArr.concat("for doc " + i);
                    }
                    messageArr = messageArr.concat(e_3.message);
                    message = messageArr.join(' ');
                    if (!ignoreMissingVariables) {
                        throw new Error(message);
                    }
                    debug(message + ", continuing");
                    return [3, 14];
                case 14:
                    i++;
                    return [3, 6];
                case 15:
                    _i++;
                    return [3, 1];
                case 16:
                    if (!objects.length) {
                        throw new Error('No objects to work with');
                    }
                    objects = objects.filter(Boolean);
                    debug("got objects: " + JSON.stringify(objects));
                    if (params.varsObj) {
                        objects = objects.map(function (object) {
                            return parsing_1.replaceTokens(object, params.varsObj, ignoreMissingVariables);
                        });
                    }
                    return [2, deep_extends_1.deepExtends.apply(void 0, objects)];
            }
        });
    });
}
exports.cobble = cobble;
